package com.winhc.phoenix.example.configuration;

import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.boot.context.properties.PropertyMapper;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Configuration;

import java.time.Duration;
import java.util.HashMap;
import java.util.Map;

/**
 * @author: XuJiakai
 * 2021/8/4 11:02
 */
@Slf4j
@Configuration
public class DynamicElasticSearchConfiguration implements ApplicationContextAware {
    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        DynamicElasticSearchProperties dynamicElasticSearchProperties = applicationContext.getBean(DynamicElasticSearchProperties.class);

        ConfigurableListableBeanFactory beanFactory = (ConfigurableListableBeanFactory) applicationContext
                .getAutowireCapableBeanFactory();

        Map<String, RestClient> map1 = new HashMap<>(12);
        Map<String, RestHighLevelClient> map2 = new HashMap<>(12);

        for (Map.Entry<String, ElasticSearchRestClientProperties> entry : dynamicElasticSearchProperties.getRest().entrySet()) {
            String beanName = entry.getKey();
            ElasticSearchRestClientProperties properties = entry.getValue();

            HttpHost[] hosts = properties.getUris().stream().map(HttpHost::create).toArray(HttpHost[]::new);
            RestClientBuilder builder = RestClient.builder(hosts);
            PropertyMapper map = PropertyMapper.get();
            map.from(properties::getUsername).whenHasText().to((username) -> {
                CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
                Credentials credentials = new UsernamePasswordCredentials(properties.getUsername(),
                        properties.getPassword());
                credentialsProvider.setCredentials(AuthScope.ANY, credentials);
                builder.setHttpClientConfigCallback(
                        (httpClientBuilder) -> httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider));
            });
            builder.setRequestConfigCallback((requestConfigBuilder) -> {
                map.from(properties::getConnectionTimeout).whenNonNull().asInt(Duration::toMillis)
                        .to(requestConfigBuilder::setConnectTimeout);
                map.from(properties::getReadTimeout).whenNonNull().asInt(Duration::toMillis)
                        .to(requestConfigBuilder::setSocketTimeout);
                return requestConfigBuilder;
            });
            RestClient restClient = elasticsearchRestClient(builder);
            RestHighLevelClient restHighLevelClient = elasticsearchRestHighLevelClient(builder, restClient);

            map1.put(beanName + "_client", restClient);
            map2.put(beanName + "_high_client", restHighLevelClient);
            beanFactory.registerSingleton(beanName + "_client", restClient);
            beanFactory.registerSingleton(beanName + "_high_client", restHighLevelClient);
            log.info("register {} successful !", beanName);
        }
        beanFactory.registerSingleton("dynamicElasticSearchClient", new DynamicElasticSearchClient(map1, map2, dynamicElasticSearchProperties.getPrimary()));
    }


    private RestClient elasticsearchRestClient(RestClientBuilder builder) {
        return builder.build();
    }

    private RestHighLevelClient elasticsearchRestHighLevelClient(RestClientBuilder restClientBuilder, RestClient restClient) {
        if (restClient != null) {
            return new RestHighLevelClient(restClient);
        }
        return new RestHighLevelClient(restClientBuilder.build());
    }

}
